@@ -26,6 +26,11 @@ setMethodS3("getFilenameExtension", "AromaCellSequenceFile", function(static, ..
   "acs";
 }, static=TRUE)
 
+setMethodS3("getDefaultExtension", "AromaCellSequenceFile", function(static, ...) {
+  "acs";
+}, static=TRUE, protected=TRUE);
+
+
 setMethodS3("getExtensionPattern", "AromaCellSequenceFile", function(static, ...) {
   "[.](acs)$";
 }, static=TRUE, protected=TRUE)
